library resource_future;

import 'package:flutter/material.dart';
import 'package:resource_result/resource_result.dart';

/// Uses flutter widget [FutureBuilder] to handle a [Future]<[Resource]<T>>
/// and handle the different states of it differently
/// through the passed functions [success], [loadingIndicator], [handleError]
class ResourceFutureBuilder<T> extends StatelessWidget {
  final Future<Resource<T>>? future;
  final Resource<T>? initialData;
  final Widget Function(BuildContext context, T data) success;
  final WidgetBuilder loadingIndicator;

  /// Defaults to [handleErrorAsText]
  Widget? Function(BuildContext context, Error error)? handleError;

  ResourceFutureBuilder(
      {required this.future,
      this.initialData,
      required this.success,
      required this.loadingIndicator,
      this.handleError,
      Key? key})
      : super(key: key);

  Text handleErrorAsText(_, Error error) => Text("Error: ${error.message}",
      style: const TextStyle(color: Colors.red, backgroundColor: Colors.white));

  @override
  Widget build(BuildContext context) {
    handleError ??= handleErrorAsText;
    return FutureBuilder<Resource<T>>(
        key: key,
        future: future,
        initialData: initialData ?? Loading(null),
        builder: (context, snapshot) {
          if (snapshot.connectionState == ConnectionState.waiting) {
            return loadingIndicator(context);
          }
          if (snapshot.hasData) {
            final resource = snapshot.data!;
            if (resource is Success) return success(context, resource.data!);
            if (resource is Loading) return loadingIndicator(context);
            final errorWidget = handleError!(context, (snapshot.data?.error)!);
            if (errorWidget != null) return errorWidget;
          }
          final errorWidget = handleError!(
              context, Error("Error in loading future ${snapshot.error}"));
          if (errorWidget != null) return errorWidget;
          return const SizedBox();
        });
  }
}
